@@ -29,30 +29,41 @@ from datetime import timedelta
 
 import mib, snmp
 
-class Type:
+class Type(object):
     """Base class for all types"""
 
     consume = 0                 # Consume all suboid if built from OID
 
-    def __init__(self, entity, value):
+    def __new__(cls, entity, value):
         """Create a new typed value
 
         @param entity: L{mib.Entity} instance
         @param value: value to set
+        @return: an instance of the new typed value
         """
-        self.value = 0          # To avoid some recursive loop
         if not isinstance(entity, mib.Entity):
             raise TypeError("%r not a mib.Entity instance" % entity)
-        if entity.type != self.__class__:
+        if entity.type != cls:
             raise ValueError("MIB node is %r. We are %r" % (entity.type,
-                                                            self.__class))
-        self.entity = entity
-        if isinstance(value, Type):
-            self.set(value.value)
-        else:
-            self.set(value)
+                                                            cls))
 
-    def set(self, value):
+        if not isinstance(value, Type):
+            value = cls._internal(entity, value)
+        else:
+            value = cls._internal(entity, value._value)
+        if issubclass(cls, str):
+            self = str.__new__(cls, value)
+        elif issubclass(cls, long):
+            self = long.__new__(cls, value)
+        else:
+            self = object.__new__(cls)
+        self._value = value
+        self.entity = entity
+        return self
+
+    @classmethod
+    def _internal(cls, entity, value):
+        """Get internal value for a given value."""
         raise NotImplementedError
 
     def pack(self):
@@ -85,84 +96,23 @@ class Type:
         """
         raise NotImplementedError
 
-    def display(self):
-        return str(self)
-
-    def __str__(self):
-        return str(self.value)
-
-    def __repr__(self):
-        try:
-            return '<%s: %s>' % (self.__class__.__name__,
-                                 self.display())
-        except:
-            return '<%s ????>' % self.__class__.__name__
-
-class IpAddress(Type):
-    """Class for IP address"""
-
-    def set(self, value):
-        if type(value) in [list, tuple]:
-            value = ".".join([str(a) for a in value])
-        try:
-            value = socket.inet_ntoa(socket.inet_aton(value))
-        except:
-            raise ValueError("%r is not a valid IP" % value)
-        self.value = [int(a) for a in value.split(".")]
-
-    def pack(self):
-        return (snmp.ASN_IPADDRESS,
-                socket.inet_aton(".".join(["%d" % x for x in self.value])))
-
-    def toOid(self):
-        return tuple(self.value)
-
-    @classmethod
-    def fromOid(cls, entity, oid):
-        if len(oid) < 4:
-            raise ValueError("%r is too short for an IP address" % (oid,))
-        return (4, cls(entity, oid[:4]))
-
-    def __str__(self):
-        return ".".join([str(a) for a in self.value])
-
-    def __cmp__(self, other):
-        if not isinstance(other, IpAddress):
-            try:
-                other = IpAddress(self.entity, other)
-            except:
-                raise NotImplementedError
-        if self.value == other.value:
-            return 0
-        if self.value < other.value:
-            return -1
-        return 1
-
-    def __getitem__(self, nb):
-        return self.value[nb]
-
-class String(Type):
-    """Class for any string"""
-
-    def set(self, value):
-        self.value = str(value)
-
-    def pack(self):
-        return (snmp.ASN_OCTET_STR, self.value)
-
     @classmethod
     def _fixedOrImplied(cls, entity):
-        """Determine if the given entity is a fixed-len string or an
-        implied var-len string.
+        """Determine if the given entity is fixed-len or implied.
+
+        This function is an helper that is used for String and
+        Oid. When converting a variable-length type to an OID, we need
+        to prefix it by its len or not depending of what the MIB say.
 
         @param entity: entity to check
-        @return: C{fixed} if it is fixed-len, C{implied} if implied var-len, C{False} otherwise
+        @return: C{fixed} if it is fixed-len, C{implied} if implied var-len,
+           C{False} otherwise
         """
-        if not(entity.ranges) or type(entity.ranges) is not tuple:
+        if entity.ranges and type(entity.ranges) is not tuple:
             # Fixed length
             return "fixed"
 
-        # We have a variable-len string. We need to know if it is impled.
+        # We have a variable-len string/oid. We need to know if it is implied.
         try:
             table = entity.table
         except:
@@ -175,13 +125,80 @@ class String(Type):
             return False
         return "implied"
 
+    def display(self):
+        return str(self)
+
+    def __str__(self):
+        return str(self._value)
+
+    def __repr__(self):
+        try:
+            return '<%s: %s>' % (self.__class__.__name__,
+                                 self.display())
+        except:
+            return '<%s ????>' % self.__class__.__name__
+
+class IpAddress(Type):
+    """Class for IP address"""
+
+    @classmethod
+    def _internal(cls, entity, value):
+        if isinstance(value, list) or isinstance(value, tuple):
+            value = ".".join([str(a) for a in value])
+        try:
+            value = socket.inet_ntoa(socket.inet_aton(value))
+        except:
+            raise ValueError("%r is not a valid IP" % value)
+        return [int(a) for a in value.split(".")]
+
+    def pack(self):
+        return (snmp.ASN_IPADDRESS,
+                socket.inet_aton(".".join(["%d" % x for x in self._value])))
+
+    def toOid(self):
+        return tuple(self._value)
+
+    @classmethod
+    def fromOid(cls, entity, oid):
+        if len(oid) < 4:
+            raise ValueError("%r is too short for an IP address" % (oid,))
+        return (4, cls(entity, oid[:4]))
+
+    def __str__(self):
+        return ".".join([str(a) for a in self._value])
+
+    def __cmp__(self, other):
+        if not isinstance(other, IpAddress):
+            try:
+                other = IpAddress(self.entity, other)
+            except:
+                raise NotImplementedError
+        if self._value == other._value:
+            return 0
+        if self._value < other._value:
+            return -1
+        return 1
+
+    def __getitem__(self, nb):
+        return self._value[nb]
+
+class String(Type, str):
+    """Class for any string"""
+
+    @classmethod
+    def _internal(cls, entity, value):
+        return str(value)
+
+    def pack(self):
+        return (snmp.ASN_OCTET_STR, self._value)
+
     def toOid(self):
         # To convert properly to OID, we need to know if it is a
         # fixed-len string, an implied string or a variable-len
         # string.
         if self._fixedOrImplied(self.entity):
-            return tuple(ord(a) for a in self.value)
-        return tuple([len(self.value)] + [ord(a) for a in self.value])
+            return tuple(ord(a) for a in self._value)
+        return tuple([len(self._value)] + [ord(a) for a in self._value])
 
     @classmethod
     def fromOid(cls, entity, oid):
@@ -205,14 +222,14 @@ class String(Type):
         return (l+1, cls(entity,"".join([chr(x) for x in oid[1:(l+1)]])))
 
     def _display(self, fmt):
-        i = 0               # Position in self.value
+        i = 0               # Position in self._value
         j = 0               # Position in fmt
         result = ""
-        while i < len(self.value):
+        while i < len(self._value):
             if j < len(fmt):
                 # repeater
                 if fmt[j] == "*":
-                    repeat = ord(self.value[i])
+                    repeat = ord(self._value[i])
                     j += 1
                     i += 1
                 else:
@@ -242,7 +259,7 @@ class String(Type):
                     term = ""
             # building
             for r in range(repeat):
-                bytes = self.value[i:i+length]
+                bytes = self._value[i:i+length]
                 i += length
                 if format in ['o', 'x', 'd']:
                     if length > 8:
@@ -269,57 +286,43 @@ class String(Type):
     def display(self):
         if self.entity.fmt:
             return self._display(self.entity.fmt)
-        if "\\x" not in repr(self.value):
-            return self.value
-        return "0x" + " ".join([("0%s" % hex(ord(a))[2:])[-2:] for a in self.value])
-
-    def __str__(self):
-        return self.value
-
-    def __getattr__(self, attr):
-        # Ugly hack to be like an string
-        return getattr(str(self), attr)
+        if "\\x" not in repr(self._value):
+            return self._value
+        return "0x" + " ".join([("0%s" % hex(ord(a))[2:])[-2:] for a in self._value])
 
     def __ior__(self, value):
-        nvalue = [ord(u) for u in self.value]
-        if type(value) not in [tuple, list]:
+        nvalue = [ord(u) for u in self._value]
+        if not isinstance(value, tuple) and not isinstance(value, list):
             value = [value]
         for v in value:
-            if isinstance(v, Integer):
-                v = int(v)
-            if type(v) is not int:
+            if not isinstance(v, int) and not isinstance(v, long):
                 raise NotImplementedError(
                     "on string, bit-operation are limited to integers")
             if len(nvalue) < v/8 + 1:
-                nvalue.extend([0] * (v/8 + 1 - len(self.value)))
+                nvalue.extend([0] * (v/8 + 1 - len(self._value)))
             nvalue[v/8] |= 1 << (7-v%8)
-        self.value = "".join([chr(i) for i in nvalue])
-        return self
+        return self.__class__(self.entity, "".join([chr(i) for i in nvalue]))
 
     def __isub__(self, value):
-        nvalue = [ord(u) for u in self.value]
-        if type(value) not in [tuple, list]:
+        nvalue = [ord(u) for u in self._value]
+        if not isinstance(value, tuple) and not isinstance(value, list):
             value = [value]
         for v in value:
-            if isinstance(v, Integer):
-                v = int(v)
-            if type(v) is not int:
+            if not isinstance(v, int) and not isinstance(v, long):
                 raise NotImplementedError(
                     "on string, bit-operation are limited to integers")
             if len(nvalue) < v/8 + 1:
                 continue
             nvalue[v/8] &= ~(1 << (7-v%8))
-        self.value = "".join([chr(i) for i in nvalue])
+        return self.__class__(self.entity, "".join([chr(i) for i in nvalue]))
         return self
 
     def __and__(self, value):
-        nvalue = [ord(u) for u in self.value]
-        if type(value) not in [tuple, list]:
+        nvalue = [ord(u) for u in self._value]
+        if not isinstance(value, tuple) and not isinstance(value, list):
             value = [value]
         for v in value:
-            if isinstance(v, Integer):
-                v = int(v)
-            if type(v) is not int:
+            if not isinstance(v, int) and not isinstance(v, long):
                 raise NotImplementedError(
                     "on string, bit-operation are limited to integers")
             if len(nvalue) < v/8 + 1:
@@ -328,29 +331,30 @@ class String(Type):
                 return False
         return True
 
-class Integer(Type):
+class Integer(Type, long):
     """Class for any integer"""
 
-    def set(self, value):
-        self.value = long(value)
+    @classmethod
+    def _internal(cls, entity, value):
+        return long(value)
 
     def pack(self):
-        if self.value >= (1L << 64):
+        if self._value >= (1L << 64):
             raise OverflowError("too large to be packed")
-        if self.value >= (1L << 32):
+        if self._value >= (1L << 32):
             # Pack in a 64 bit counter
             return (snmp.ASN_OCTET_STR,
                     struct.pack("LL",
-                                self.value/(1L << 32),
-                                self.value%(1L << 32)))
-        if self.value >= 0:
-            return (snmp.ASN_INTEGER, struct.pack("L", self.value))
-        if self.value >= -(1L << 31):
-            return (snmp.ASN_INTEGER, struct.pack("l", self.value))
+                                self._value/(1L << 32),
+                                self._value%(1L << 32)))
+        if self._value >= 0:
+            return (snmp.ASN_INTEGER, struct.pack("L", self._value))
+        if self._value >= -(1L << 31):
+            return (snmp.ASN_INTEGER, struct.pack("l", self._value))
         raise OverflowError("too small to be packed")
 
     def toOid(self):
-        return (self.value,)
+        return (self._value,)
 
     @classmethod
     def fromOid(cls, entity, oid):
@@ -358,26 +362,17 @@ class Integer(Type):
             raise ValueError("%r is too short for an integer" % (oid,))
         return (1, cls(entity, oid[0]))
 
-    def __int__(self):
-        return int(self.value)
-
-    def __long__(self):
-        return long(self.value)
-
-    def __str__(self):
-        return str(self.value)
-
     def display(self):
         if self.entity.fmt:
             if self.entity.fmt[0] == "x":
-                return hex(self.value)
+                return hex(self._value)
             if self.entity.fmt[0] == "o":
-                return oct(self.value)
+                return oct(self._value)
             if self.entity.fmt[0] == "b":
-                if self.value == 0:
+                if self._value == 0:
                     return "0"
-                if self.value > 0:
-                    v = self.value
+                if self._value > 0:
+                    v = self._value
                     r = ""
                     while v > 0:
                         r = str(v%2) + r
@@ -387,35 +382,47 @@ class Integer(Type):
                     len(self.entity.fmt) > 2 and \
                     self.entity.fmt[1] == "-":
                 dec = int(self.entity.fmt[2:])
-                result = str(self.value)
+                result = str(self._value)
                 if len(result) < dec + 1:
                     result = "0"*(dec + 1 - len(result)) + result
                 return "%s.%s" % (result[:-2], result[-2:])
-        return str(self.value)
+        return str(self._value)
 
-    def __getattr__(self, attr):
-        # Ugly hack to be like an integer
-        return getattr(self.value, attr)
+class Unsigned32(Integer):
+    def pack(self):
+        if self._value >= (1L << 32):
+            raise OverflowError("too large to be packed")
+        if self._value < 0:
+            raise OverflowError("too small to be packed")
+        return (snmp.ASN_UNSIGNED, struct.pack("L", self._value))
 
-class Enum(Type):
+class Unsigned64(Integer):
+    def pack(self):
+        if self._value >= (1L << 64):
+            raise OverflowError("too large to be packed")
+        if self._value < 0:
+            raise OverflowError("too small to be packed")
+        return (snmp.ASN_COUNTER64, struct.pack("LL",
+            self._value/(1L << 32), self._value%(1L << 32)))
+
+class Enum(Integer):
     """Class for enumeration"""
 
-    def set(self, value):
-        if value in self.entity.enum:
-            self.value = value
-            return
-        for (k, v) in self.entity.enum.iteritems():
+    @classmethod
+    def _internal(cls, entity, value):
+        if value in entity.enum:
+            return value
+        for (k, v) in entity.enum.iteritems():
             if (v == value):
-                self.value = k
-                return
-        raise ValueError("%r is not a valid value for %s" % (value,
-                                                             self.entity))
+                return k
+        try:
+            return long(value)
+        except:
+            raise ValueError("%r is not a valid value for %s" % (value,
+                                                                 entity))
 
     def pack(self):
-        return (snmp.ASN_INTEGER, struct.pack("l", self.value))
-
-    def toOid(self):
-        return (self.value,)
+        return (snmp.ASN_INTEGER, struct.pack("l", self._value))
 
     @classmethod
     def fromOid(cls, entity, oid):
@@ -429,40 +436,48 @@ class Enum(Type):
                 other = self.__class__(self.entity, other)
             except:
                 raise NotImplementedError
-        return self.value == other.value
+        return self._value == other._value
+
+    def __ne__(self, other):
+        return not(self.__eq__(other))
 
     def __str__(self):
-        return "%s(%d)" % (self.entity.enum[self.value], self.value)
+        if self._value in self.entity.enum:
+            return "%s(%d)" % (self.entity.enum[self._value], self._value)
+        else:
+            return "%d" % self._value
+
+    def display(self):
+        return str(self)
 
 class Oid(Type):
     """Class for OID"""
 
-    def set(self, value):
-        if type(value) in [list, tuple]:
-            self.value = tuple([int(v) for v in value])
-        elif type(value) is str:
-            self.value = tuple([int(i) for i in value.split(".") if i])
+    @classmethod
+    def _internal(cls, entity, value):
+        if isinstance(value, list) or isinstance(value, tuple):
+            return tuple([int(v) for v in value])
+        elif isinstance(value, str):
+            return tuple([int(i) for i in value.split(".") if i])
         elif isinstance(value, mib.Entity):
-            self.value = tuple(value.oid)
+            return tuple(value.oid)
         else:
             raise TypeError("don't know how to convert %r to OID" % value)
             
     def pack(self):
-        return (snmp.ASN_OBJECT_ID, "".join([struct.pack("l", x) for x in self.value]))
+        return (snmp.ASN_OBJECT_ID, "".join([struct.pack("l", x) for x in self._value]))
 
     def toOid(self):
-        return self.value
+        if self._fixedOrImplied(self.entity):
+            return self._value
+        return tuple([len(self._value)] + list(self._value))
 
     @classmethod
     def fromOid(cls, entity, oid):
-        try:
-            table = entity.table
-        except:
-            raise NotImplementedError("%r is not an index of a table" % entity)
-        indexes = [str(a) for a in table.index]
-        if str(entity) not in indexes:
-            raise NotImplementedError("%r is not an index of a table" % entity)
-        if str(entity) != indexes[-1] or not table.implied:
+        if cls._fixedOrImplied(entity) == "fixed":
+            # A fixed OID? We don't like this. Provide a real example.
+            raise ValueError("%r seems to be a fixed-len OID index. Odd." % entity)
+        if not cls._fixedOrImplied(entity):
             # This index is not implied. We need the len
             if len(oid) < 1:
                 raise ValueError("%r is too short for a not implied index" % entity)
@@ -475,14 +490,14 @@ class Oid(Type):
             return (len(oid), cls(entity, oid))
 
     def __str__(self):
-        return ".".join([str(x) for x in self.value])
+        return ".".join([str(x) for x in self._value])
                  
     def __cmp__(self, other):
         if not isinstance(other, Oid):
             other = Oid(self.entity, other)
-        if tuple(self.value) == tuple(other.value):
+        if tuple(self._value) == tuple(other._value):
             return 0
-        if self.value > other.value:
+        if self._value > other._value:
             return 1
         return -1
 
@@ -490,42 +505,45 @@ class Oid(Type):
         """Test if item is a sub-oid of this OID"""
         if not isinstance(item, Oid):
             item = Oid(self.entity, item)
-        return tuple(item.value[:len(self.value)]) == \
-            tuple(self.value[:len(self.value)])
+        return tuple(item._value[:len(self._value)]) == \
+            tuple(self._value[:len(self._value)])
 
 
 class Boolean(Enum):
     """Class for boolean"""
 
-    def set(self, value):
+    @classmethod
+    def _internal(cls, entity, value):
         if type(value) is bool:
             if value:
-                Enum.set(self, "true")
+                return Enum._internal(entity, "true")
             else:
-                Enum.set(self, "false")
+                return Enum._internal(entity, "false")
         else:
-            Enum.set(self, value)
+            return Enum._internal(entity, value)
 
-    def __getattr__(self, attr):
-        if self.value == 1:
-            return getattr(True, attr)
-        return getattr(False, attr)
+    def __nonzero__(self):
+        if self._value == 1:
+            return True
+        else:
+            return False
 
 class Timeticks(Type):
     """Class for timeticks"""
 
-    def set(self, value):
-        if type(value) is int or type(value) is long:
+    @classmethod
+    def _internal(cls, entity, value):
+        if isinstance(value, int) or isinstance(value, long):
             # Value in centiseconds
-            self.value = timedelta(0, value/100.)
+            return timedelta(0, value/100.)
         elif isinstance(value, timedelta):
-            self.value = value
+            return value
         else:
             raise TypeError("dunno how to handle %r (%s)" % (value, type(value)))
 
     def __int__(self):
-        return self.value.days*3600*24*100 + self.value.seconds*100 + \
-            self.value.microseconds/10000
+        return self._value.days*3600*24*100 + self._value.seconds*100 + \
+            self._value.microseconds/10000
 
     def toOid(self):
         return (int(self),)
@@ -542,21 +560,23 @@ class Timeticks(Type):
                             int(self)))
 
     def __str__(self):
-        return str(self.value)
+        return str(self._value)
 
     def __cmp__(self, other):
-        if type(other) is int:
+        if isinstance(other, int) or isinstance(other, long):
             other = timedelta(0, other/100.)
         elif not isinstance(other, timedelta):
             raise NotImplementedError("only compare to int or timedelta")
-        if self.value == other:
+        if self._value == other:
             return 0
-        if self.value < other:
+        if self._value < other:
             return -1
         return 1
 
     def __eq__(self, other):
         return self.__cmp__(other) == 0
+    def __ne__(self, other):
+        return self.__cmp__(other) != 0
     def __lt__(self, other):
         return self.__cmp__(other) < 0
     def __gt__(self, other):
@@ -565,36 +585,36 @@ class Timeticks(Type):
 class Bits(Type):
     """Class for bits"""
 
-    def set(self, value):
+    @classmethod
+    def _internal(cls, entity, value):
         bits = []
         tryalternate = False
-        if type(value) is str:
+        if isinstance(value, str):
             for i,x in enumerate(value):
                 if ord(x) == 0:
                     continue
                 for j in range(8):
                     if ord(x) & (1 << (7-j)):
-                        if j not in self.entity.enum:
+                        if j not in entity.enum:
                             tryalternate = True
                             break
                         bits.append(j)
                 if tryalternate:
                     break
-            self.value = bits
             if not tryalternate:
-                return
+                return bits
             else:
                 bits = []
-        if type(value) not in [tuple, list]:
+        elif not isinstance(value, tuple) and not isinstance(value, list):
             value = [value]
         for v in value:
             found = False
-            if v in self.entity.enum:
+            if v in entity.enum:
                 if v not in bits:
                     bits.append(v)
                     found = True
             else:
-                for (k, t) in self.entity.enum.iteritems():
+                for (k, t) in entity.enum.iteritems():
                     if (t == v):
                         if k not in bits:
                             bits.append(k)
@@ -603,58 +623,58 @@ class Bits(Type):
             if not found:
                 raise ValueError("%r is not a valid bit value" % v)
         bits.sort()
-        self.value = bits
+        return bits
 
     def pack(self):
         string = []
-        for b in self.value:
+        for b in self._value:
             if len(string) < b/16 + 1:
                 string.extend([0]*(b/16 - len(string)+1))
             string[b/16] |= 1 << (7 - b%16)
         return (snmp.ASN_OCTET_STR, "".join([chr(x) for x in string]))
 
     def __eq__(self, other):
-        if type(other) is str:
+        if isinstance(other, str):
             other = [other]
         if not isinstance(other, Bits):
             other = Bits(self.entity, other)
-        return self.value == other.value
+        return self._value == other._value
 
     def __str__(self):
         result = []
-        for b in self.value:
+        for b in self._value:
             result.append("%s(%d)" % (self.entity.enum[b], b))
         return ", ".join(result)
 
     def __and__(self, other):
-        if type(other) is str:
+        if isinstance(other, str):
             other = [other]
         if not isinstance(other, Bits):
             other = Bits(self.entity, other)
-        for o in other.value:
-            if o not in self.value:
+        for o in other._value:
+            if o not in self._value:
                 return False
         return True
 
     def __ior__(self, other):
-        if type(other) is str:
+        if isinstance(other, str):
             other = [other]
         if not isinstance(other, Bits):
             other = Bits(self.entity, other)
-        for o in other.value:
-            if o not in self.value:
-                self.value.append(o)
-        self.value.sort()
+        for o in other._value:
+            if o not in self._value:
+                self._value.append(o)
+        self._value.sort()
         return self
 
     def __isub__(self, other):
-        if type(other) is str:
+        if isinstance(other, str):
             other = [other]
         if not isinstance(other, Bits):
             other = Bits(self.entity, other)
-        for o in other.value:
-            if o in self.value:
-                self.value.remove(o)
+        for o in other._value:
+            if o in self._value:
+                self._value.remove(o)
         return self
 
 def build(mibname, entity, value):
