@@ -38,15 +38,6 @@ import os
 import atexit
 import code
 from datetime import timedelta
-try:
-    import rlcompleter
-    import readline
-except ImportError:
-    readline = None
-try:
-    from IPython.Shell import IPShellEmbed
-except ImportError:
-    IPShellEmbed = None
 
 import manager
 from config import conf
@@ -84,14 +75,37 @@ def interact():
         execfile(sys.argv[0], local)
         return
 
-    if IPShellEmbed and conf.ipython:
-        argv = ["-prompt_in1", "Snimpy [\\#]> ",
-                "-prompt_out", "Snimpy [\\#]: "]
-        if conf.ipythonprofile:
-            argv += ["-profile", conf.ipythonprofile]
-        shell = IPShellEmbed(argv=argv,
-                             banner=banner, user_ns=local)
-        shell.IP.InteractiveTB.tb_offset += 1 # Not interested by traceback in this module
+    try:
+        import rlcompleter
+        import readline
+    except ImportError:
+        readline = None
+    try:
+        try:
+            # ipython >= 0.11
+            from IPython.frontend.terminal.embed import InteractiveShellEmbed
+            from IPython.config.loader import Config
+            cfg = Config()
+            cfg.InteractiveShellEmbed.prompt_in1="Snimpy [\\#]> "
+            cfg.InteractiveShellEmbed.prompt_out="Snimpy [\\#]: "
+            if conf.ipythonprofile:
+              cfg.InteractiveShellEmbed.profile=conf.ipythonprofile
+            shell = InteractiveShellEmbed(config=cfg, banner1=banner, user_ns=local)
+            shell.InteractiveTB.tb_offset += 1 # Not interested by traceback in this module
+        except ImportError:
+            # ipython < 0.11
+            from IPython.Shell import IPShellEmbed
+            argv = ["-prompt_in1", "Snimpy [\\#]> ",
+                    "-prompt_out", "Snimpy [\\#]: "]
+            if conf.ipythonprofile:
+                argv += ["-profile", conf.ipythonprofile]
+            shell = IPShellEmbed(argv=argv,
+                                 banner=banner, user_ns=local)
+            shell.IP.InteractiveTB.tb_offset += 1 # Not interested by traceback in this module
+    except ImportError:
+        shell = None
+
+    if shell and conf.ipython:
         shell()
     else:
         if readline:
